part of tecfy_core_client_package;

class DashboardStatisticsComponenet extends StatefulWidget {
  /// item type
  final DashboardItemBase type;

  /// dashboard componenet
  DashboardStatisticsComponenet(this.type);
  @override
  _DashboardStatisticsComponenetState createState() =>
      _DashboardStatisticsComponenetState();
}

class _DashboardStatisticsComponenetState
    extends State<DashboardStatisticsComponenet> {
  _onItemTab(type) async {
    if (type.onClick == null || type.onClick.isEmpty) return;
    if (type.onClick.startsWith('EntityList:')) {
      AppNavigator.navigateTo(context,
          () => EntityListPage(type.onClick.replaceAll('EntityList:', '')));
      return;
    }
    if (type.onClick.startsWith('Report:')) {
      var result =
          await TecfyBasicApp.apiService.httpGetDynamic('Report/GetList');
      var reportName = type.onClick.replaceAll('Report:', '');
      var report = result.data
          .firstWhere((s) => s['dbName'] == reportName, orElse: () => null);
      if (report == null) return;
      ReportDataService.onOpenReport(context, report);
      return;
    }
  }

  Future<dynamic> _getData() async {
    var type = widget.type;
    if (type.value != null) return type.value;
    if (type.value == -1) return null;
    type.value = -1;
    var result = await TecfyBasicApp.apiService
        .httpGetDynamic('Dashboard/GetData/${type.dbName}');
    if (!result.success) {
      result.data = jsonDecode(
          ConfigService.getValueString('DashboardType${type.dbName}'));
      result.success = result.data != null;
    }
    //print(result.toString());
    if (result.success) {
      ConfigService.setValueString('DashboardType${type.dbName}',
          jsonEncode(result.data, toEncodable: EntityService.customEncode));
      type.value = result.data;
    } else {
      type.value = '0';
    }
    setState(() {});
    return type.value;
  }

  _getWidth() {
    var fullwidth = LayoutService.getWidth(context);
    if (LayoutService.isMobile(context)) {
      return (fullwidth - 20) / 2;
    }
    fullwidth -= (ThemeService.menuWidth + 40);
    if (fullwidth > ThemeService.containerWidth)
      fullwidth = ThemeService.containerWidth - 40;
    if (fullwidth < 500) return fullwidth / 2;
    if (fullwidth < 680) return fullwidth / 4;
    if (fullwidth < 740) return fullwidth / 5;
    return fullwidth / 6;
  }

  @override
  Widget build(BuildContext context) {
    return Container(
        //constraints: BoxConstraints(minWidth: 250),
        width: _getWidth(),
        child: GestureDetector(
            onTap: () => _onItemTab(widget.type),
            child: Card(
                elevation: 2,
                margin: EdgeInsets.all(10),
                child: SizedBox(
                  height: 90,
                  child: Column(
                    children: <Widget>[
                      SizedBox(height: 10),
                      FittedBox(
                          fit: BoxFit.fitWidth,
                          child: Padding(
                              padding: EdgeInsets.all(3),
                              child: TecfyText(
                                widget.type.title,
                                textAlign: TextAlign.center,
                                maxLines: 1,
                                softWrap: false,
                                overflow: TextOverflow.visible,
                              ))),
                      Expanded(child: SizedBox()),
                      FutureBuilder<dynamic>(
                        future: _getData(),
                        builder: (c, snap) => FittedBox(
                            fit: BoxFit.fitWidth,
                            child: Text(
                                widget.type.value != null
                                    ? widget.type.value
                                        .toString()
                                        .trim()
                                        .replaceAll(
                                            'SAR',
                                            LocalConfigService.isArabic
                                                ? 'ريال'
                                                : 'SAR')
                                        .replaceAll(
                                            'EGP',
                                            LocalConfigService.isArabic
                                                ? 'جنيه'
                                                : 'EGP')
                                    : '${AppLocale.translate('Loading')}..',
                                style: TextStyle(fontSize: 15))),
                      ),
                      SizedBox(height: 10),
                    ],
                  ),
                ))));
  }
}
