part of tecfy_core_client_package;

class PhotoProperty extends StatefulWidget {
  final Map<String, dynamic>? fileData;

  /// label string
  final String? label;
  final String? description;

  /// is required
  final bool isRequired;

  /// field id
  final String? fileId;

  /// field id
  final int maxSize;

  /// on changed
  final void Function(Map<String, dynamic>)? onChanged;
  PhotoProperty(
      {this.fileData,
      this.label,
      this.description,
      this.maxSize = 200,
      this.onChanged,
      this.fileId,
      this.isRequired = false}); //: assert(fileId != null );
  @override
  _PhotoPropertyState createState() => _PhotoPropertyState();
}

class _PhotoPropertyState extends State<PhotoProperty> {
  dynamic _photo;

  _onChosePhoto() async {
    var maxSize = widget.maxSize;
    if (maxSize == 0) maxSize = 200;
    _photo = await ImageService().getImage(
      maxHeight: maxSize.toDouble(),
      maxWidth: maxSize.toDouble(),
      //imageQuality: 100,
    );

    if (_photo != null) {
      AppNavigator.showLoading(context);
      //print('File Length : ${photo.data.length}');
      // show loading and upload the photo
      var result = await TecfyBasicApp.apiService.httpPostFile('Attachment',
          (await _photo!.readAsBytes()), path.basename(_photo!.name));
      Navigator.pop(context);
      if (result.success) {
        widget.onChanged?.call({'id': result.data});
        AppNavigator.showMessage(
            context, 'تم رفع الصورة بنجاح', MessageType.success);
      } else {
        print('XXXXXXXXXXXXXXXXXXXX Image Upload Error: ${result.message}');
        AppNavigator.showMessage(
            context, 'خطأ اثناء رفع الصوره', MessageType.error,
            message: result.message);
        _photo = null;
      }
      //print(result.toString());
    }
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return Padding(
        padding: EdgeInsets.all(10),
        child: Row(
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            GestureDetector(
              onTap: _onChosePhoto,
              child: Tooltip(
                  message: widget.description ?? widget.label ?? '',
                  child: _photo != null
                      ? CircleAvatar(
                          backgroundImage: Image.file(File(_photo!.path)).image,
                          radius: 55.0,
                        )
                      : widget.fileData != null || widget.fileId != null
                          ? CircleAvatar(
                              backgroundImage: ImageService.imageProvider(
                                  widget.fileData != null &&
                                          widget.fileData!['id'] != null
                                      ? widget.fileData!['id']
                                      : widget.fileId),
                              radius: 55.0)
                          :
                          // Image.asset('assets/images/profile_pic.png',height: 110.0)
                          Column(
                              children: <Widget>[
                                Icon(Icons.photo_size_select_actual),
                                TecfyText(
                                    '${widget.label ?? 'Select Image'}${widget.isRequired ? '*' : ''}')
                              ],
                            )),
            ),
          ],
        ));
  }
}
